@@ -1,6 +1,6 @@
 #!/usr/bin/python
 #
-# Copyright 2018-2020 Polyaxon, Inc.
+# Copyright 2018-2021 Polyaxon, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@ import os
 
 from concurrent import futures
 from io import BytesIO
+from typing import List
 
 from botocore.exceptions import ClientError
 
@@ -257,7 +258,7 @@ class S3Service(AWSService, StoreMixin):
         bytes_data,
         key,
         bucket_name=None,
-        overwrite=False,
+        overwrite=True,
         encrypt=False,
         acl=None,
     ):
@@ -300,7 +301,7 @@ class S3Service(AWSService, StoreMixin):
         string_data,
         key,
         bucket_name=None,
-        overwrite=False,
+        overwrite=True,
         encrypt=False,
         acl=None,
         encoding="utf-8",
@@ -336,7 +337,7 @@ class S3Service(AWSService, StoreMixin):
         filename,
         key,
         bucket_name=None,
-        overwrite=False,
+        overwrite=True,
         encrypt=False,
         acl=None,
         use_basename=True,
@@ -406,12 +407,13 @@ class S3Service(AWSService, StoreMixin):
         dirname,
         key,
         bucket_name=None,
-        overwrite=False,
+        overwrite=True,
         encrypt=False,
         acl=None,
         use_basename=True,
         workers=0,
         last_time=None,
+        exclude: List[str] = None,
     ):
         """
         Uploads a local directory to S3.
@@ -427,7 +429,9 @@ class S3Service(AWSService, StoreMixin):
                 by S3 and will be stored in an encrypted form while at rest in S3.
             acl: `str`. ACL to use for uploading, e.g. "public-read".
             use_basename: `bool`. whether or not to use the basename of the directory.
-            last_time: `datetime`. if provided, it will only upload the file if changed after last_time.
+            last_time: `datetime`. if provided,
+                        it will only upload the file if changed after last_time.
+            exclude: `list`. List of paths to excludes.
         """
         if not bucket_name:
             bucket_name, key = self.parse_s3_url(key)
@@ -439,7 +443,7 @@ class S3Service(AWSService, StoreMixin):
 
         # Turn the path to absolute paths
         dirname = os.path.abspath(dirname)
-        with get_files_in_path_context(dirname) as files:
+        with get_files_in_path_context(dirname, exclude=exclude) as files:
             for f in files:
 
                 # If last time is provided we check if we should re-upload the file
@@ -545,7 +549,7 @@ class S3Service(AWSService, StoreMixin):
         # Delete directories
         for prefix in sorted(results["prefixes"]):
             prefix = os.path.join(key, prefix)
-            # Download files under
+            # Deletes files under
             self.delete(key=prefix, bucket_name=bucket_name)
 
         pool, future_results = self.init_pool(workers)
