@@ -1,6 +1,6 @@
 #!/usr/bin/python
 #
-# Copyright 2018-2020 Polyaxon, Inc.
+# Copyright 2018-2021 Polyaxon, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,7 +16,7 @@
 
 import sys
 
-from polyaxon.constants import DEFAULT
+from polyaxon.constants.globals import DEFAULT
 from polyaxon.env_vars.getters import get_entity_info
 from polyaxon.env_vars.getters.user import get_local_owner
 from polyaxon.exceptions import PolyaxonClientException
@@ -26,15 +26,13 @@ from polyaxon.utils.formatting import Printer
 
 
 def get_project_error_message(owner, project):
-    message_context = ""
-    if owner:
-        message_context += " <owner: {}>".format(owner)
-    if project:
-        message_context += " <project: {}>".format(project)
-    if message_context:
-        message_context = " Context:{}".format(message_context)
     if not owner or not project:
-        return "Please provide a valid project with owner.{}".format(message_context)
+        return (
+            "Please provide a valid project. "
+            "Context: <owner: {}> - <project: {}>".format(
+                owner or "Missing", project or "Missing"
+            )
+        )
 
 
 def get_project_or_local(project=None, is_cli: bool = False):
@@ -49,9 +47,16 @@ def get_project_or_local(project=None, is_cli: bool = False):
             raise PolyaxonClientException(error_message)
 
     if project:
-        owner, project_name = get_entity_info(project)
+        try:
+            owner, project_name = get_entity_info(project)
+        except Exception as e:
+            if is_cli:
+                Printer.print_error("Please provide a valid project name.\n%s" % e)
+                sys.exit(1)
+            else:
+                raise e
     else:
-        project = get_local_project()
+        project = get_local_project(is_cli=is_cli)
 
         owner, project_name = project.owner, project.name
 
