@@ -1,6 +1,6 @@
 #!/usr/bin/python
 #
-# Copyright 2018-2020 Polyaxon, Inc.
+# Copyright 2018-2021 Polyaxon, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,7 +19,7 @@ import sys
 from polyaxon_sdk import V1Project
 
 from polyaxon.managers.base import BaseConfigManager
-from polyaxon.utils import constants
+from polyaxon.utils import cli_constants
 from polyaxon.utils.formatting import Printer
 
 
@@ -37,7 +37,7 @@ class ProjectConfigManager(BaseConfigManager):
         if not project:
             Printer.print_error(
                 "No project was found, please initialize a project."
-                " {}".format(constants.INIT_COMMAND)
+                " {}".format(cli_constants.INIT_COMMAND)
             )
             sys.exit(1)
 
