@@ -1,6 +1,6 @@
 #!/usr/bin/python
 #
-# Copyright 2018-2020 Polyaxon, Inc.
+# Copyright 2018-2021 Polyaxon, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,7 +16,7 @@
 
 from marshmallow import fields, validate
 
-from polyaxon.polyflow.actions import ActionSchema
+from polyaxon.polyflow.builds import BuildSchema
 from polyaxon.polyflow.cache import CacheSchema
 from polyaxon.polyflow.hooks import HookSchema
 from polyaxon.polyflow.plugins import PluginsSchema
@@ -31,12 +31,12 @@ class BaseComponentSchema(BaseCamelSchema):
     name = fields.Str(validate=validate.Regexp(regex=NAME_REGEX), allow_none=True)
     description = fields.Str(allow_none=True)
     tags = fields.List(fields.Str(), allow_none=True)
-    presets = fields.List(fields.Str(allow_none=True))
+    presets = RefOrObject(fields.List(fields.Str(allow_none=True)))
     queue = RefOrObject(fields.Str(allow_none=True))
     cache = fields.Nested(CacheSchema, allow_none=True)
     termination = fields.Nested(TerminationSchema, allow_none=True)
     plugins = fields.Nested(PluginsSchema, allow_none=True)
-    actions = fields.List(fields.Nested(ActionSchema), allow_none=True)
+    build = fields.Nested(BuildSchema, allow_none=True)
     hooks = fields.List(fields.Nested(HookSchema), allow_none=True)
     is_approved = fields.Bool(allow_none=True)
 
@@ -58,7 +58,7 @@ class BaseComponent(BaseConfig):
         "cache",
         "termination",
         "plugins",
-        "actions",
+        "build",
         "hooks",
         "isApproved",
     ]
