@@ -1,6 +1,6 @@
 #!/usr/bin/python
 #
-# Copyright 2018-2020 Polyaxon, Inc.
+# Copyright 2018-2021 Polyaxon, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@ import polyaxon_sdk
 
 from marshmallow import fields, validate
 
+from polyaxon.polyflow import BuildSchema
 from polyaxon.polyflow.component.base import BaseComponent, BaseComponentSchema
 from polyaxon.polyflow.io import IOSchema
 from polyaxon.polyflow.references import RefMixin
@@ -27,8 +28,9 @@ from polyaxon.polyflow.templates import TemplateMixinConfig, TemplateMixinSchema
 
 class ComponentSchema(BaseComponentSchema, TemplateMixinSchema):
     kind = fields.Str(allow_none=True, validate=validate.Equal("component"))
-    inputs = fields.Nested(IOSchema, allow_none=True, many=True)
-    outputs = fields.Nested(IOSchema, allow_none=True, many=True)
+    inputs = fields.List(fields.Nested(IOSchema), allow_none=True)
+    outputs = fields.List(fields.Nested(IOSchema), allow_none=True)
+    build = fields.Nested(BuildSchema, allow_none=True)
     run = fields.Nested(RunSchema, required=True)
 
     @staticmethod
@@ -67,18 +69,11 @@ class V1Component(
         cache: [V1Cache](/docs/automation/helpers/cache/), optional
         termination: [V1Termination](/docs/core/specification/termination/), optional
         plugins: [V1Plugins](/docs/core/specification/plugins/), optional
-        actions: List[[V1Action](/docs/automation/extensions/actions/)], optional
-        hooks: List[[V1Hook](/docs/automation/extensions/hooks/)], optional
+        build: [V1Build](/docs/automation/builds/), optional
+        hooks: List[[V1Hook](/docs/automation/hooks/)], optional
         inputs: [V1IO](/docs/core/specification/io/), optional
         outputs: [V1IO](/docs/core/specification/io/), optional
-        run: Union[[V1Job](/docs/experimentation/jobs/),
-             [V1Service](/docs/experimentation/services/),
-             [V1TFJob](/docs/experimentation/distributed/tf-jobs/),
-             [V1PytorchJob](/docs/experimentation/distributed/pytorch-jobs/),
-             [V1MPIJob](/docs/experimentation/distributed/mpi-jobs/),
-             [V1Spark](/docs/experimentation/distributed/spark-jobs/),
-             [V1Dask](/docs/experimentation/distributed/dask-jobs/),
-             [V1Dag](/docs/automation/flow-engine/specification/)]
+        run: Union[[V1Job](/docs/experimentation/jobs/), [V1Service](/docs/experimentation/services/), [V1TFJob](/docs/experimentation/distributed/tf-jobs/), [V1PytorchJob](/docs/experimentation/distributed/pytorch-jobs/), [V1MPIJob](/docs/experimentation/distributed/mpi-jobs/), [V1Spark](/docs/experimentation/distributed/spark-jobs/), [V1Dask](/docs/experimentation/distributed/dask-jobs/), [V1Dag](/docs/automation/flow-engine/specification/)]  # noqa
         template: [V1Template](/docs/core/specification/template/), optional
 
     ## YAML usage
@@ -99,6 +94,7 @@ class V1Component(
     >>>   hooks:
     >>>   inputs:
     >>>   outputs:
+    >>>   build:
     >>>   run:
     >>>   isApproved:
     >>>   template:
@@ -107,7 +103,9 @@ class V1Component(
     ## Python usage
 
     ```python
-    >>> from polyaxon.polyflow import V1Cache, V1Component, V1IO, V1Plugins, V1Termination
+    >>> from polyaxon.polyflow import (
+    >>>     V1Build, V1Cache, V1Component, V1Hook, V1IO, V1Plugins, V1Termination
+    >>> )
     >>> component = V1Component(
     >>>     name="test",
     >>>     description="test",
@@ -117,10 +115,10 @@ class V1Component(
     >>>     cache=V1Cache(...),
     >>>     termination=V1Termination(...),
     >>>     plugins=V1Plugins(...),
-    >>>     actions=[V1Action(...)],
     >>>     hooks=[V1Hook(...)],
-    >>>     inputs=V1IO(...),
-    >>>     outputs=V1IO(...),
+    >>>     inputs=[V1IO(...)],
+    >>>     outputs=[V1IO(...)],
+    >>>     build=V1Build(...),
     >>>     run=...
     >>> )
     ```
@@ -332,22 +330,38 @@ class V1Component(
     >>>   ...
     ```
 
+    ### build
+
+    > **Note**: Please check [V1Build](/docs/automation/builds/) for more details.
+
+    This section defines if this component should build a container before starting the main logic.
+    If the build section is provided, Polyaxon will set the main operation to a pending state
+    until the build is done and then it will use the resulting docker image
+    for starting the main container.
+
+    ```yaml
+    >>> component:
+    >>>   ...
+    >>>   build:
+    >>>     hubRef: kaniko
+    >>>   ...
+    ```
+
     ### run
 
     This is the section that defines the runtime of the component:
-        * [V1Job](/docs/experimentation/jobs/): for running batch jobs, model training experiments,
-                                                data processing jobs, ...
-        * [V1Service](/docs/experimentation/services/): for running tensorboards, notebooks,
-                                                        streamlit, custom services or an API.
-        * [V1TFJob](/docs/experimentation/distributed/tf-jobs/): for running distributed
-                                                                 Tensorflow training job.
-        * [V1PytorchJob](/docs/experimentation/distributed/pytorch-jobs/): for running distributed
-                                                                           Pytorch training job.
-        * [V1MPIJob](/docs/experimentation/distributed/mpi-jobs/): for running distributed
-                                                                   MPI job.
-        * [V1Spark](/docs/experimentation/distributed/spark-jobs/): for running a spark Application.
-        * [V1Dask](/docs/experimentation/distributed/dask-jobs/): for running a Dask job.
-        * [V1Dag](/docs/automation/flow-engine/specification/): for running a DAG/workflow.
+     * [V1Job](/docs/experimentation/jobs/): for running batch jobs, model training experiments,
+       data processing jobs, ...
+     * [V1Service](/docs/experimentation/services/): for running tensorboards, notebooks,
+       streamlit, custom services or an API.
+     * [V1TFJob](/docs/experimentation/distributed/tf-jobs/): for running distributed
+       Tensorflow training job.
+     * [V1PytorchJob](/docs/experimentation/distributed/pytorch-jobs/): for running distributed
+       Pytorch training job.
+     * [V1MPIJob](/docs/experimentation/distributed/mpi-jobs/): for running distributed MPI job.
+     * [V1Spark](/docs/experimentation/distributed/spark-jobs/): for running a spark Application.
+     * [V1Dask](/docs/experimentation/distributed/dask-jobs/): for running a Dask job.
+     * [V1Dag](/docs/automation/flow-engine/specification/): for running a DAG/workflow.
 
     ### isApproved
 
