@@ -1,6 +1,6 @@
 #!/usr/bin/python
 #
-# Copyright 2018-2020 Polyaxon, Inc.
+# Copyright 2018-2021 Polyaxon, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,17 +16,19 @@
 
 import polyaxon_sdk
 
-from marshmallow import fields, validate
+from marshmallow import fields, validate, validates_schema
 
 from polyaxon.k8s import k8s_schemas
 from polyaxon.schemas.base import BaseCamelSchema, BaseConfig
 from polyaxon.schemas.fields.swagger import SwaggerField
+from polyaxon.utils.sanitizers import sanitize_string_dict
+from polyaxon.utils.signal_decorators import check_partial
 
 
 class EnvironmentSchema(BaseCamelSchema):
-    labels = fields.Dict(values=fields.Str(), keys=fields.Str(), allow_none=True)
-    annotations = fields.Dict(values=fields.Str(), keys=fields.Str(), allow_none=True)
-    node_selector = fields.Dict(values=fields.Str(), keys=fields.Str(), allow_none=True)
+    labels = fields.Dict(allow_none=True)
+    annotations = fields.Dict(allow_none=True)
+    node_selector = fields.Dict(allow_none=True)
     affinity = SwaggerField(cls=k8s_schemas.V1Affinity, allow_none=True)
     tolerations = fields.List(
         SwaggerField(cls=k8s_schemas.V1Toleration), allow_none=True
@@ -39,6 +41,7 @@ class EnvironmentSchema(BaseCamelSchema):
     security_context = SwaggerField(cls=k8s_schemas.V1SecurityContext, allow_none=True)
     image_pull_secrets = fields.List(fields.Str(), allow_none=True)
     host_network = fields.Bool(allow_none=True)
+    host_pid = fields.Bool(allow_none=True, data_key="hostPID")
     dns_policy = fields.Str(allow_none=True)
     dns_config = SwaggerField(cls=k8s_schemas.V1PodDNSConfig, allow_none=True)
     scheduler_name = fields.Str(allow_none=True)
@@ -52,6 +55,21 @@ class EnvironmentSchema(BaseCamelSchema):
     def schema_config():
         return V1Environment
 
+    @validates_schema
+    @check_partial
+    def validate_param(self, values, **kwargs):
+        labels = values.get("labels")
+        if labels:
+            values["labels"] = sanitize_string_dict(labels)
+        annotations = values.get("annotations")
+        if annotations:
+            values["annotations"] = sanitize_string_dict(annotations)
+        node_selector = sanitize_string_dict(values.get("nodeSelector"))
+        if node_selector:
+            values["nodeSelector"] = sanitize_string_dict(node_selector)
+
+        return values
+
 
 class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
     """The environment section allows to alter the
@@ -72,6 +90,7 @@ class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
         security_context: V1SecurityContext, optional
         image_pull_secrets: List[str], optional
         host_network: bool, optional
+        host_pid: bool, optional
         dns_policy: str, optional
         dns_config: V1PodDNSConfig, optional
         scheduler_name: str, optional
@@ -94,6 +113,7 @@ class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
     >>>   securityContext:
     >>>   imagePullSecrets:
     >>>   hostNetwork:
+    >>>   hostPID:
     >>>   dnsPolicy:
     >>>   dnsConfig:
     >>>   schedulerName:
@@ -126,6 +146,7 @@ class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
     >>>     security_context=V1SecurityContext(...),
     >>>     image_pull_secrets=["secret1", "secret2", ...],
     >>>     host_network=False,
+    >>>     host_pid=False,
     >>>     dns_policy="Default",
     >>>     dns_config=V1PodDNSConfig(...),
     >>>     scheduler_name="name",
@@ -142,8 +163,8 @@ class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
     From [Kubernetes docs](https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/)  # noqa
 
     > Labels are key/value pairs that are attached to objects, such as pods.
-      Labels are intended to be used to specify identifying attributes of objects that
-      are meaningful and relevant to users, but do not directly imply semantics to the core system.
+    > Labels are intended to be used to specify identifying attributes of objects that
+    > are meaningful and relevant to users, but do not directly imply semantics to the core system.
 
     Polyaxon injects several labels to all operations it manages,
     users can leverage those labels or extend them.
@@ -160,7 +181,7 @@ class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
     From [Kubernetes docs](https://kubernetes.io/docs/concepts/overview/working-with-objects/annotations/)  # noqa
 
     > You can use Kubernetes annotations to attach arbitrary non-identifying metadata to objects.
-      Clients such as tools and libraries can retrieve this metadata.
+    > Clients such as tools and libraries can retrieve this metadata.
 
     ```yaml
     >>> environment:
@@ -174,10 +195,10 @@ class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
     From [Kubernetes docs](https://kubernetes.io/docs/concepts/scheduling-eviction/assign-pod-node/#nodeselector)  # noqa
 
     > nodeSelector is the simplest recommended form of node selection constraint.
-      nodeSelector is a field of PodSpec. It specifies a map of key-value pairs.
-      For the pod to be eligible to run on a node, the node must have each of
-      the indicated key-value pairs as labels (it can have additional labels as well).
-      The most common usage is one key-value pair.
+    > nodeSelector is a field of PodSpec. It specifies a map of key-value pairs.
+    > For the pod to be eligible to run on a node, the node must have each of
+    > the indicated key-value pairs as labels (it can have additional labels as well).
+    > The most common usage is one key-value pair.
 
     ```yaml
     >>> environment:
@@ -191,7 +212,7 @@ class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
 
     > The affinity/anti-affinity feature, greatly expands the types of constraints you can express.
 
-    The affinity to use for the scheduling the job.
+    The affinity to use for scheduling the job.
 
     ```yaml
     >>> environment:
@@ -207,7 +228,7 @@ class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
     From [Kubernetes docs](https://kubernetes.io/docs/concepts/scheduling-eviction/taint-and-toleration/)  # noqa
 
     > Tolerations are applied to pods, and allow (but do not require)
-      the pods to schedule onto nodes with matching taints.
+    > the pods to schedule onto nodes with matching taints.
 
     ```yaml
     >>> environment:
@@ -222,10 +243,10 @@ class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
     From [Kubernetes docs](https://kubernetes.io/docs/concepts/scheduling-eviction/assign-pod-node/#nodename)  # noqa
 
     > nodeName is the simplest form of node selection constraint,
-      but due to its limitations it is typically not used. nodeName is a field of PodSpec.
-      If it is non-empty, the scheduler ignores the pod and the kubelet running on the named
-      node tries to run the pod. Thus, if nodeName is provided in the PodSpec,
-      it takes precedence over the above methods for node selection.
+    > but due to its limitations it is typically not used. nodeName is a field of PodSpec.
+    > If it is non-empty, the scheduler ignores the pod and the kubelet running on the named
+    > node tries to run the pod. Thus, if nodeName is provided in the PodSpec,
+    > it takes precedence over the above methods for node selection.
 
     ```yaml
     >>> environment:
@@ -264,8 +285,8 @@ class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
     From [Kubernetes docs](https://kubernetes.io/docs/concepts/services-networking/add-entries-to-pod-etc-hosts-with-host-aliases/)  # noqa
 
     > Adding entries to a Pod’s /etc/hosts file provides Pod-level override of hostname resolution
-      when DNS and other options are not applicable. In 1.7,
-      users can add these custom entries with the HostAliases field in PodSpec.
+    > when DNS and other options are not applicable. In 1.7,
+    > users can add these custom entries with the HostAliases field in PodSpec.
 
     ```yaml
     >>> environment:
@@ -288,7 +309,7 @@ class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
     >>>   securityContext:
     >>>     runAsUser: 1000
     >>>     runAsGroup: 3000
-    >>>     fsGroup: 2000
+    >>>     runAsNonRoot: true
     ```
 
     ### imagePullSecrets
@@ -296,9 +317,9 @@ class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
     From [Kubernetes docs](https://kubernetes.io/docs/concepts/containers/images/#specifying-imagepullsecrets-on-a-pod)  # noqa
 
     > ImagePullSecrets is a list of references to secrets in the same namespace
-      to use for pulling any images in pods that reference this ServiceAccount.
-      ImagePullSecrets are distinct from Secrets because Secrets can be mounted in the pod,
-      but ImagePullSecrets are only accessed by the kubelet.
+    > to use for pulling any images in pods that reference this ServiceAccount.
+    > ImagePullSecrets are distinct from Secrets because Secrets can be mounted in the pod,
+    > but ImagePullSecrets are only accessed by the kubelet.
 
     ```yaml
     >>> environment:
@@ -310,24 +331,37 @@ class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
     From [Kubernetes docs](https://kubernetes.io/docs/concepts/policy/pod-security-policy/#host-namespaces)  # noqa
 
     > Controls whether the pod may use the node network namespace.
-      Doing so gives the pod access to the loopback device, services listening on localhost,
-      and could be used to snoop on network activity of other pods on the same node.
+    > Gives the pod access to the loopback device, services listening on localhost,
+    > and could be used to snoop on network activity of other pods on the same node.
 
     ```yaml
     >>> environment:
     >>>   hostNetwork: false
     ```
 
+    ### hostPID
+
+    From [Kubernetes docs](https://kubernetes.io/docs/concepts/policy/pod-security-policy/#host-namespaces)  # noqa
+
+    > Controls whether the pod containers can share the host process ID namespace.
+    > Note that when paired with ptrace this can be used to escalate privileges outside
+    > of the container (ptrace is forbidden by default).
+
+    ```yaml
+    >>> environment:
+    >>>   hostPID: false
+    ```
+
     ### dnsPolicy
 
     From [Kubernetes docs](https://kubernetes.io/docs/concepts/services-networking/dns-pod-service/#pods-dns-policy)  # noqa
 
     > Set DNS policy for the pod.
-      Defaults to "ClusterFirst".
-      Valid values are 'ClusterFirstWithHostNet', 'ClusterFirst', 'Default' or 'None'.
-      DNS parameters given in DNSConfig will be merged with the policy selected with DNSPolicy.
-      To have DNS options set along with hostNetwork, you have to specify DNS policy
-      explicitly to 'ClusterFirstWithHostNet'.
+    > Defaults to "ClusterFirst".
+    > Valid values are 'ClusterFirstWithHostNet', 'ClusterFirst', 'Default' or 'None'.
+    > DNS parameters given in DNSConfig will be merged with the policy selected with DNSPolicy.
+    > To have DNS options set along with hostNetwork, you have to specify DNS policy
+    > explicitly to 'ClusterFirstWithHostNet'.
 
     ```yaml
     >>> environment:
@@ -358,9 +392,9 @@ class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
 
     From [Kubernetes docs](https://kubernetes.io/docs/tasks/administer-cluster/configure-multiple-schedulers/)  # noqa
 
-    > If specified, the pod will be dispatched by specified scheduler.
-      Or it will be dispatched by workflow scope scheduler if specified.
-      If neither specified, the pod will be dispatched by default scheduler.
+    > If specified, the pod will be dispatched by the specified scheduler.
+    > Or it will be dispatched by workflow scope scheduler if specified.
+    > If neither specified, the pod will be dispatched by default scheduler.
 
     ```yaml
     >>> environment:
@@ -372,9 +406,9 @@ class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
     From [Kubernetes docs](https://kubernetes.io/docs/concepts/configuration/pod-priority-preemption/)  # noqa
 
     > Pods can have priority.
-      Priority indicates the importance of a Pod relative to other Pods.
-      If a Pod cannot be scheduled, the scheduler tries to preempt (evict)
-      lower priority Pods to make scheduling of the pending Pod possible.
+    > Priority indicates the importance of a Pod relative to other Pods.
+    > If a Pod cannot be scheduled, the scheduler tries to preempt (evict)
+    > lower priority Pods to make scheduling of the pending Pod possible.
 
     ```yaml
     >>> environment:
@@ -386,9 +420,9 @@ class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
     From [Kubernetes docs](https://kubernetes.io/docs/concepts/configuration/pod-priority-preemption/)  # noqa
 
     > Pods can have priority.
-      Priority indicates the importance of a Pod relative to other Pods.
-      If a Pod cannot be scheduled, the scheduler tries to preempt (evict)
-      lower priority Pods to make scheduling of the pending Pod possible.
+    > Priority indicates the importance of a Pod relative to other Pods.
+    > If a Pod cannot be scheduled, the scheduler tries to preempt (evict)
+    > lower priority Pods to make scheduling of the pending Pod possible.
 
     ```yaml
     >>> environment:
@@ -400,7 +434,7 @@ class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
     From [Kubernetes docs](https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle/#restart-policy)  # noqa
 
     > A PodSpec has a restartPolicy field with possible values Always, OnFailure, and Never.
-      The default value is Always.
+    > The default value is Always.
 
 
     ```yaml
@@ -423,6 +457,7 @@ class V1Environment(BaseConfig, polyaxon_sdk.V1Environment):
         "securityContext",
         "imagePullSecrets",
         "hostNetwork",
+        "hostPID",
         "dnsPolicy",
         "dnsConfig",
         "schedulerName",
