@@ -1,6 +1,6 @@
 #!/usr/bin/python
 #
-# Copyright 2018-2020 Polyaxon, Inc.
+# Copyright 2018-2021 Polyaxon, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,53 +18,47 @@ import polyaxon_sdk
 
 from marshmallow import fields, validate
 
-from polyaxon.containers.names import MAIN_JOB_CONTAINER
-from polyaxon.k8s import k8s_schemas
 from polyaxon.polyflow.early_stopping import EarlyStoppingSchema
+from polyaxon.polyflow.matrix.base import BaseSearchConfig
 from polyaxon.polyflow.matrix.kinds import V1MatrixKind
-from polyaxon.polyflow.matrix.params import MatrixSchema
-from polyaxon.schemas.base import BaseCamelSchema, BaseConfig
+from polyaxon.polyflow.matrix.params import HpParamSchema
+from polyaxon.polyflow.matrix.tuner import TunerSchema
+from polyaxon.schemas.base import BaseCamelSchema
 from polyaxon.schemas.fields.ref_or_obj import RefOrObject
-from polyaxon.schemas.fields.swagger import SwaggerField
 
 
 class IterativeSchema(BaseCamelSchema):
     kind = fields.Str(allow_none=True, validate=validate.Equal(V1MatrixKind.ITERATIVE))
-    num_iterations = RefOrObject(
+    max_iterations = RefOrObject(
         fields.Int(required=True, validate=validate.Range(min=1)), required=True
     )
-    concurrency = fields.Int(allow_none=True)
+    concurrency = RefOrObject(fields.Int(allow_none=True))
     params = fields.Dict(
-        keys=fields.Str(), values=fields.Nested(MatrixSchema), allow_none=True
+        keys=fields.Str(), values=fields.Nested(HpParamSchema), allow_none=True
     )
     seed = RefOrObject(fields.Int(allow_none=True))
-    container = SwaggerField(
-        cls=k8s_schemas.V1Container,
-        defaults={"name": MAIN_JOB_CONTAINER},
-        allow_none=True,
-    )
-    early_stopping = fields.Nested(EarlyStoppingSchema, many=True, allow_none=True)
+    tuner = fields.Nested(TunerSchema, allow_none=True)
+    early_stopping = fields.List(fields.Nested(EarlyStoppingSchema), allow_none=True)
 
     @staticmethod
     def schema_config():
         return V1Iterative
 
 
-class V1Iterative(BaseConfig, polyaxon_sdk.V1Iterative):
-    """To build a custom optimization algorithm, this interface let's you create an iterative
+class V1Iterative(BaseSearchConfig, polyaxon_sdk.V1Iterative):
+    """To build a custom optimization algorithm, this interface lets you create an iterative
     process for creating suggestions and training your model based on those suggestions
 
-    The iterative process expect a user defined container that will generate the suggestions for
+    The iterative process expect a user defined a tuner that will generate the suggestions for
     running the component.
 
     Args:
         kind: str, should be equal `iterative`
-        num_iterations: int
-        params: List[Dict[str,
-        [params](/docs/automation/optimization-engine/params/)]]
+        max_iterations: int
+        params: List[Dict[str, [params](/docs/automation/optimization-engine/params/)]]
         concurrency: int, optional
         seed: int, optional
-        container: [Kubernetes Container](https://kubernetes.io/docs/concepts/containers/)
+        tuner: [V1Tuner](/docs/automation/optimization-engine/tuner/), optional
         early_stopping: List[[EarlyStopping](/docs/automation/helpers/early-stopping)], optional
 
     ## YAML usage
@@ -74,9 +68,9 @@ class V1Iterative(BaseConfig, polyaxon_sdk.V1Iterative):
     >>>   kind: iterative
     >>>   concurrency:
     >>>   params:
-    >>>   numIterations:
+    >>>   maxIterations:
     >>>   seed:
-    >>>   container:
+    >>>   tuner:
     >>>   earlyStopping:
     ```
 
@@ -85,15 +79,20 @@ class V1Iterative(BaseConfig, polyaxon_sdk.V1Iterative):
     ```python
     >>> from polyaxon.k8s import k8s_schemas
     >>> from polyaxon.polyflow import (
-    >>>     V1Iterative, V1HpLogSpace, V1HpUniform, V1FailureEarlyStopping, V1MetricEarlyStopping
+    >>>     V1Iterative,
+    >>>     V1HpLogSpace,
+    >>>     V1HpUniform,
+    >>>     V1FailureEarlyStopping,
+    >>>     V1MetricEarlyStopping,
+    >>>     V1Tuner,
     >>> )
     >>> matrix = V1Iterative(
-    >>>   num_iterations=20,
+    >>>   max_iterations=20,
     >>>   concurrency=2,
     >>>   seed=23,
     >>>   params={"param1": V1HpLogSpace(...), "param2": V1HpUniform(...), ... },
     >>>   early_stopping=[V1FailureEarlyStopping(...), V1MetricEarlyStopping(...)],
-    >>>   container=k8s_schemas.V1Container(name="my-suggestion-container", ...)
+    >>>   tuner=V1Tuner(hub_ref="org/my-suggestion-component")
     >>> )
     ```
 
@@ -137,8 +136,8 @@ class V1Iterative(BaseConfig, polyaxon_sdk.V1Iterative):
     [params generators](/docs/automation/optimization-engine/params/).
 
     > The parameters generated will be validated against
-    the component's inputs/outputs definition to check that the values
-    can be passed and have valid types.
+    > the component's inputs/outputs definition to check that the values
+    > can be passed and have valid types.
 
     ```yaml
     >>> matrix:
@@ -152,14 +151,14 @@ class V1Iterative(BaseConfig, polyaxon_sdk.V1Iterative):
     >>>        value: ...
     ```
 
-    ### numIterations
+    ### maxIterations
 
     Maximum number of iterations to run the process of \\-> suggestions -> training ->\\
 
     ```yaml
     >>> matrix:
     >>>   kind: iterative
-    >>>   numIterations: 5
+    >>>   maxIterations: 5
     ```
 
     ### seed
@@ -189,15 +188,16 @@ class V1Iterative(BaseConfig, polyaxon_sdk.V1Iterative):
     For more details please check the
     [early stopping section](/docs/automation/helpers/early-stopping/).
 
-    ### container
+    ### tuner
 
-    The container will container the logic for creating new suggestions
-    to run the main container in the compoent with different new params combinations.
+    The tuner reference definition (with a component hub reference) to use.
+    The component contains the logic for creating new suggestions.
 
     ```yaml
     >>> matrix:
     >>>   kind: iterative
-    >>>   container: ...
+    >>>   tuner:
+    >>>     hubRef: acme/suggestion-logic:v1
     ```
 
     ## Example
@@ -212,10 +212,9 @@ class V1Iterative(BaseConfig, polyaxon_sdk.V1Iterative):
     >>> matrix:
     >>>   kind: iterative
     >>>   concurrency: 10
-    >>>   numIterations: 5
-    >>>   container:
-    >>>     name: my-suggestion-logic
-    >>>     commmand: ...
+    >>>   maxIterations: 5
+    >>>   tuner:
+    >>>     hubRef: my-suggestion-component
     >>>   params:
     >>>     lr:
     >>>       kind: logspace
@@ -259,4 +258,20 @@ class V1Iterative(BaseConfig, polyaxon_sdk.V1Iterative):
 
     IDENTIFIER = V1MatrixKind.ITERATIVE
     SCHEMA = IterativeSchema
-    REDUCED_ATTRIBUTES = ["params", "seed", "container", "earlyStopping", "concurrency"]
+    REDUCED_ATTRIBUTES = [
+        "maxIterations",
+        "params",
+        "seed",
+        "tuner",
+        "earlyStopping",
+        "concurrency",
+    ]
+
+    def create_iteration(self, iteration: int = None) -> int:
+        if iteration is None:
+            return 0
+        return iteration + 1
+
+    def should_reschedule(self, iteration):
+        """Return a boolean to indicate if we need to reschedule another iteration."""
+        return iteration < self.max_iterations
