@@ -1,6 +1,6 @@
 #!/usr/bin/python
 #
-# Copyright 2018-2020 Polyaxon, Inc.
+# Copyright 2018-2021 Polyaxon, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,18 +13,22 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from typing import Dict
 
 import polyaxon_sdk
 
 from marshmallow import ValidationError, fields, validate, validates_schema
 
+from polyaxon.polyflow.builds import BuildSchema, V1Build
 from polyaxon.polyflow.component.component import ComponentSchema
+from polyaxon.polyflow.hooks import V1Hook
 from polyaxon.polyflow.operations.base import BaseOp, BaseOpSchema
-from polyaxon.polyflow.params import ParamSchema
+from polyaxon.polyflow.params import ParamSchema, V1Param
 from polyaxon.polyflow.references import V1DagRef, V1HubRef, V1PathRef, V1UrlRef
 from polyaxon.polyflow.run.patch import validate_run_patch
 from polyaxon.polyflow.templates import TemplateMixinConfig, TemplateMixinSchema
 from polyaxon.schemas.patch_strategy import V1PatchStrategy
+from polyaxon.utils.signal_decorators import check_partial
 
 
 class OperationSchema(BaseOpSchema, TemplateMixinSchema):
@@ -32,6 +36,7 @@ class OperationSchema(BaseOpSchema, TemplateMixinSchema):
     params = fields.Dict(
         keys=fields.Str(), values=fields.Nested(ParamSchema), allow_none=True
     )
+    build = fields.Nested(BuildSchema, allow_none=True)
     run_patch = fields.Dict(keys=fields.Str(), values=fields.Raw(), allow_none=True)
     hub_ref = fields.Str(allow_none=True)
     dag_ref = fields.Str(allow_none=True)
@@ -48,6 +53,7 @@ class OperationSchema(BaseOpSchema, TemplateMixinSchema):
         return V1Operation
 
     @validates_schema
+    @check_partial
     def validate_run_patch(self, data, **kwargs):
         component = data.get("component")
         run_patch = data.get("run_patch")
@@ -57,6 +63,7 @@ class OperationSchema(BaseOpSchema, TemplateMixinSchema):
         validate_run_patch(run_patch=run_patch, kind=component.run.kind)
 
     @validates_schema
+    @check_partial
     def validate_reference(self, data, **kwargs):
         if data.get("is_preset"):
             return
@@ -105,7 +112,7 @@ class V1Operation(BaseOp, TemplateMixinConfig, polyaxon_sdk.V1Operation):
     Args:
         version: str
         kind: str, should be equal to `operation`
-        patch_strategy: str, optional, defaults to post_merge.
+        patch_strategy: str, optional, defaults to post_merge
         is_preset: bool, optional
         is_approved: bool, optional
         name: str, optional
@@ -117,29 +124,16 @@ class V1Operation(BaseOp, TemplateMixinConfig, polyaxon_sdk.V1Operation):
         termination: [V1Termination](/docs/core/specification/termination/), optional
         plugins: [V1Plugins](/docs/core/specification/plugins/), optional
         params: Dict[str, [V1Param](/docs/core/specification/params/)], optional
-        schedule: Union[[V1CronSchedule](/docs/automation/schedules/cron/)
-                  [V1IntervalSchedule](/docs/automation/schedules/interval/),
-                  [V1DateTimeSchedule](/docs/automation/schedules/datetime/)], optional
-        events: List[[str](/docs/automation/extensions/events/)], optional
-        actions: List[[V1Action](/docs/automation/extensions/actions/)], optional
-        hooks: List[[V1Hook](/docs/automation/extensions/hooks/)], optional
-        matrix: Union[[V1Mapping](/docs/automation/mapping/),
-                  [V1GridSearch](/docs/automation/optimization-engine/grid-search/),
-                  [V1RandomSearch](/docs/automation/optimization-engine/random-search/),
-                  [V1Hyperband](/docs/automation/optimization-engine/hyperband/),
-                  [V1Bayes](/docs/automation/optimization-engine/bayesian-optimization/),
-                  [V1Hyperopt](/docs/automation/optimization-engine/hyperopt/),
-                  [V1Iterative](/docs/automation/optimization-engine/iterative/)], optional
-        dependencies:
-            [dependencies](/docs/automation/flow-engine/specification/#dependencies),
-            optional
-        trigger: [trigger](/docs/automation/flow-engine/specification/#trigger),
-                 optional
-        conditions: [conditions](/docs/automation/flow-engine/specification/#conditions),
-                    optional
-        skip_on_upstream_skip:
-            [skip_on_upstream_skip](/docs/automation/flow-engine/skiponupstreamskip/),
-            optional
+        schedule: Union[[V1CronSchedule](/docs/automation/schedules/cron/), [V1IntervalSchedule](/docs/automation/schedules/interval/), [V1DateTimeSchedule](/docs/automation/schedules/datetime/)], optional  # noqa
+        events: List[[V1EventTrigger](/docs/automation/events/)], optional
+        build: [V1Build](/docs/automation/builds/), optional
+        hooks: List[[V1Hook](/docs/automation/hooks/)], optional
+        matrix: Union[[V1Mapping](/docs/automation/mapping/), [V1GridSearch](/docs/automation/optimization-engine/grid-search/), [V1RandomSearch](/docs/automation/optimization-engine/random-search/), [V1Hyperband](/docs/automation/optimization-engine/hyperband/), [V1Bayes](/docs/automation/optimization-engine/bayesian-optimization/), [V1Hyperopt](/docs/automation/optimization-engine/hyperopt/), [V1Iterative](/docs/automation/optimization-engine/iterative/)], optional  # noqa
+        joins: List[[V1Join](/docs/automation/joins/)], optional
+        dependencies: [dependencies](/docs/automation/flow-engine/flow-dependencies/#dependencies), optional  # noqa
+        trigger: [trigger](/docs/automation/flow-engine/flow-dependencies/#trigger), optional
+        conditions: [conditions](/docs/core/scheduling-strategies/conditional-scheduling/#conditional-scheduling), optional  # noqa
+        skip_on_upstream_skip: [skip_on_upstream_skip](/docs/automation/flow-engine/flow-dependencies/skiponupstreamskip/), optional  # noqa
         run_patch: Dict, optional
         hub_ref: str, optional
         dag_ref: str, optional
@@ -169,6 +163,7 @@ class V1Operation(BaseOp, TemplateMixinConfig, polyaxon_sdk.V1Operation):
     >>>   actions:
     >>>   hooks:
     >>>   params:
+    >>>   build:
     >>>   runPatch:
     >>>   hubRef:
     >>>   dagRef:
@@ -181,7 +176,7 @@ class V1Operation(BaseOp, TemplateMixinConfig, polyaxon_sdk.V1Operation):
 
     ```python
     >>> from polyaxon.polyflow import (
-    >>>     V1Action, V1Cache, V1Component, V1Hook, V1Param, V1Plugins, V1Operation, V1Termination
+    >>>     V1Build, V1Cache, V1Component, V1Hook, V1Param, V1Plugins, V1Operation, V1Termination
     >>> )
     >>> from polyaxon.schemas import V1PatchStrategy
     >>> operation = V1Operation(
@@ -195,9 +190,9 @@ class V1Operation(BaseOp, TemplateMixinConfig, polyaxon_sdk.V1Operation):
     >>>     termination=V1Termination(...),
     >>>     plugins=V1Plugins(...),
     >>>     events=["event-ref1", "event-ref2"],
-    >>>     actions=[V1Action(...)],
     >>>     hooks=[V1Hook(...)],
     >>>     outputs={"param1": V1Param(...), ...},
+    >>>     build=V1Build(...),
     >>>     component=V1Component(...),
     >>> )
     ```
@@ -245,7 +240,7 @@ class V1Operation(BaseOp, TemplateMixinConfig, polyaxon_sdk.V1Operation):
     For instance a user might want to define a scheduling
     behavior that applies to several operations.
     One way to do that is to set the environment section on every operation.
-    But sometime the same scheduling behavior makes sense for several operations and components.
+    But sometimes the same scheduling behavior makes sense for several operations and components.
     In that case, the user can define an operation preset to extract that logic:
 
     ```yaml
@@ -263,7 +258,7 @@ class V1Operation(BaseOp, TemplateMixinConfig, polyaxon_sdk.V1Operation):
     ```
 
     > **Note**: Please check this
-        [in-depth section about presets](/docs/core/scheduling-strategies/presets/).
+    > [in-depth section about presets](/docs/core/scheduling-strategies/presets/).
 
     ### name
 
@@ -300,7 +295,7 @@ class V1Operation(BaseOp, TemplateMixinConfig, polyaxon_sdk.V1Operation):
 
     ### presets
 
-    The [presets](/docs/management/ui/presets/) to use for this operation run,
+    The [presets](/docs/management/organizations/presets/) to use for this operation run,
     if provided, it will override the component's presets otherwise
     the presets of the component will be used if it exists.
 
@@ -374,7 +369,7 @@ class V1Operation(BaseOp, TemplateMixinConfig, polyaxon_sdk.V1Operation):
     The [params](/docs/core/specification/params/)  to pass to the component,
     they will be validated against the inputs/outputs.
     If a parameter is passed and the component does not define a corresponding inputs/outputs,
-    a validation error will be raised unless the param has the contextOnly flag enabled.
+    a validation error will be raised unless the param has the `contextOnly` flag enabled.
 
     ```yaml
     >>> operation:
@@ -385,6 +380,22 @@ class V1Operation(BaseOp, TemplateMixinConfig, polyaxon_sdk.V1Operation):
     >>>   ...
     ```
 
+    ### build
+
+    > **Note**: Please check [V1Build](/docs/automation/builds/) for more details.
+
+    This section defines if this operation should build a container before starting the main logic.
+    If the build section is provided, Polyaxon will set the main operation to a pending state
+    until the build is done and then it will use the resulting docker image
+    for starting the main container.
+
+    ```yaml
+    >>> operation:
+    >>>   ...
+    >>>   build:
+    >>>     hubRef: kaniko
+    >>>   ...
+
     ### runPatch
 
     The run patch provides a way to override information about the component's run section,
@@ -393,19 +404,14 @@ class V1Operation(BaseOp, TemplateMixinConfig, polyaxon_sdk.V1Operation):
     The run patch is a dictionary that can modify most of the runtime information and
     will be resolved against the corresponding run kind:
 
-        * [V1Job](/docs/experimentation/jobs/): for running batch jobs, model training experiments,
-                                                data processing jobs, ...
-        * [V1Service](/docs/experimentation/services/): for running tensorboards, notebooks,
-                                                        streamlit, custom services or an API.
-        * [V1TFJob](/docs/experimentation/distributed/tf-jobs/): for running distributed
-                                                                 Tensorflow training job.
-        * [V1PytorchJob](/docs/experimentation/distributed/pytorch-jobs/): for running distributed
-                                                                           Pytorch training job.
-        * [V1MPIJob](/docs/experimentation/distributed/mpi-jobs/): for running distributed
-                                                                   MPI job.
-        * [V1Spark](/docs/experimentation/distributed/spark-jobs/): for running a spark Application.
-        * [V1Dask](/docs/experimentation/distributed/dask-jobs/): for running a Dask job.
-        * [V1Dag](/docs/automation/flow-engine/specification/): for running a DAG/workflow.
+     * [V1Job](/docs/experimentation/jobs/): for running batch jobs, model training experiments, data processing jobs, ...  # noqa
+     * [V1Service](/docs/experimentation/services/): for running tensorboards, notebooks, streamlit, custom services or an API.  # noqa
+     * [V1TFJob](/docs/experimentation/distributed/tf-jobs/): for running distributed Tensorflow training job.  # noqa
+     * [V1PytorchJob](/docs/experimentation/distributed/pytorch-jobs/): for running distributed Pytorch training job.
+     * [V1MPIJob](/docs/experimentation/distributed/mpi-jobs/): for running distributed MPI job.  # noqa
+     * [V1Spark](/docs/experimentation/distributed/spark-jobs/): for running a spark Application.
+     * [V1Dask](/docs/experimentation/distributed/dask-jobs/): for running a Dask job.
+     * [V1Dag](/docs/automation/flow-engine/specification/): for running a DAG/workflow.
 
     For example, fi we define a generic component for running Jupyter Notebook:
 
@@ -647,3 +653,49 @@ class V1Operation(BaseOp, TemplateMixinConfig, polyaxon_sdk.V1Operation):
         run_patch.pop("kind")
         result.run_patch = run_patch
         return result
+
+    @classmethod
+    def from_hook(cls, hook: V1Hook, contexts: Dict):
+        run_patch = None
+        if hook.connection:
+            run_patch = {"connections": [hook.connection]}
+        params = hook.params
+        # Extend params with
+        if not hook.disable_defaults:
+            contexts = contexts or {}
+            params = params or {}
+            for k, v in contexts.items():
+                params[k] = V1Param(value=v, context_only=True)
+
+        return cls(
+            run_patch=run_patch,
+            hub_ref=hook.hub_ref,
+            presets=hook.presets,
+            queue=hook.queue,
+            params=params,
+        )
+
+    @classmethod
+    def from_build(cls, build: V1Build, contexts: Dict = None):
+        # Extend params with
+        contexts = contexts or {}
+        params = build.params or {}
+        for k, v in contexts.items():
+            params[k] = V1Param(value=v, context_only=True)
+
+        destination = params.get("destination") or V1Param()
+        if not destination.value:
+            destination.value = "{{ globals.project_name }}:{{ globals.uuid }}"
+        if not destination.connection or build.connection:
+            destination.connection = build.connection
+        params["destination"] = destination
+
+        return cls(
+            run_patch=build.run_patch,
+            patch_strategy=build.patch_strategy,
+            hub_ref=build.hub_ref,
+            presets=build.presets,
+            queue=build.queue,
+            cache=build.cache,
+            params=params,
+        )
