@@ -1,6 +1,6 @@
 #!/usr/bin/python
 #
-# Copyright 2018-2020 Polyaxon, Inc.
+# Copyright 2018-2021 Polyaxon, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@ import polyaxon_sdk
 
 from marshmallow import fields
 
+from polyaxon.auxiliaries.sidecar import PolyaxonSidecarContainerSchema
 from polyaxon.polyflow.notifications import NotificationSchema
 from polyaxon.schemas.base import BaseCamelSchema, BaseConfig
 
@@ -26,13 +27,19 @@ class PluginsSchema(BaseCamelSchema):
     auth = fields.Bool(allow_none=True)
     docker = fields.Bool(allow_none=True)
     shm = fields.Bool(allow_none=True)
+    mount_artifacts_store = fields.Bool(allow_none=True)
     collect_artifacts = fields.Bool(allow_none=True)
     collect_logs = fields.Bool(allow_none=True)
     collect_resources = fields.Bool(allow_none=True)
     sync_statuses = fields.Bool(allow_none=True)
     auto_resume = fields.Bool(allow_none=True)
+    external_host = fields.Bool(allow_none=True)
     log_level = fields.Str(allow_none=True)
     notifications = fields.List(fields.Nested(NotificationSchema), allow_none=True)
+    sidecar = fields.Nested(
+        PolyaxonSidecarContainerSchema,
+        allow_none=True,
+    )
 
     @staticmethod
     def schema_config():
@@ -40,9 +47,9 @@ class PluginsSchema(BaseCamelSchema):
 
 
 class V1Plugins(BaseConfig, polyaxon_sdk.V1Plugins):
-    """Plugins section provides a way to set notifications and customize extrat Polyaxon utilities.
+    """Plugins section provides a way to customize extra Polyaxon utilities.
 
-    By default Polyaxon injects some information for example an auth context
+    By default, Polyaxon injects some information for example an auth context
     and triggers some mechanisms for collecting logs and outputs.
 
     Plugins section provides more control to the end user to enable/disable some of these utilities.
@@ -51,13 +58,14 @@ class V1Plugins(BaseConfig, polyaxon_sdk.V1Plugins):
         auth: bool, optional, default: True
         docker: bool, optional, default: False
         shm: bool, optional, default: True
+        mount_artifacts_store: bool, optional, default: True
         collect_artifacts: bool, optional, default: True
         collect_logs: bool, optional, default: True
         collect_resources: bool, optional, default: True
         auto_resume: bool, optional, default: True
         sync_statuses: bool, optional, default: True
         log_level: str, optional
-        notifications: List[[V1Notification](/docs/core/specification/notifications/)]
+        sidecar: V1PolyaxonSidecarContainer, optional
 
     ## YAML usage
 
@@ -66,30 +74,33 @@ class V1Plugins(BaseConfig, polyaxon_sdk.V1Plugins):
     >>>   auth:
     >>>   docker:
     >>>   shm:
+    >>>   mountArtifactsStore:
     >>>   collectArtifacts:
     >>>   collectLogs:
     >>>   collectResources:
     >>>   autoResume:
     >>>   syncStatuses:
+    >>>   externalHost:
     >>>   logLevel:
-    >>>   notifications:
+    >>>   sidecar:
     ```
 
     ## Python usage
 
     ```python
-    >>> from polyaxon.polyflow import V1Notification, V1Plugins
+    >>> from polyaxon.polyflow import V1Plugins
     >>> plugins = V1Plugins(
     >>>     auth=False,
     >>>     docker=True,
     >>>     shm=True.
+    >>>     mount_artifacts_store=True,
     >>>     collect_artifacts=False,
     >>>     collect_logs=False,
     >>>     collect_resources=False
     >>>     auto_resume=False,
     >>>     sync_statuses=False,
+    >>>     external_host=False,
     >>>     log_level="INFO",
-    >>>     notifications=[V1Notification(...)]
     >>> )
     ```
 
@@ -101,7 +112,7 @@ class V1Plugins(BaseConfig, polyaxon_sdk.V1Plugins):
     This plugin is enabled by default in Polyaxon deployments with user management.
     </blockquote>
 
-    By default Polyaxon will create an auth context for each operation, this removes the overhead
+    By default, Polyaxon will create an auth context for each operation, this removes the overhead
     to think about how you can pass tokens to your runs,
     or the need to create secrets to load the token from during the run time.
 
@@ -146,7 +157,7 @@ class V1Plugins(BaseConfig, polyaxon_sdk.V1Plugins):
     This plugin mounts an tmpfs volume to /dev/shm.
     This will set /dev/shm size to half of the RAM of node.
     By default, /dev/shm is very small, only 64MB.
-    Some experiments/jobs will fail due to lack of share memory,
+    Some experiments/jobs will fail due to the lack of shared memory,
     such as some experiments running on Pytorch.
 
     To disable this plugin:
@@ -156,11 +167,21 @@ class V1Plugins(BaseConfig, polyaxon_sdk.V1Plugins):
     >>>   shm: false
     ```
 
+    ### mountArtifactsStore
+
+    <blockquote class="light">This plugin is disabled by default.</blockquote>
+
+    this plugin allows to request the default artifacts store and mount it to the main container
+    without adding the connection reference name to the `connections` section.
+
+    This is usually very useful than setting the `connections` section as it make the component
+    more generic and will not break if the artifacts store name changes.
+
     ### collectArtifacts
 
     <blockquote class="light">This plugin is enabled by default.</blockquote>
 
-    By default Polyaxon will collect all artifacts and outputs that you share in the
+    By default, Polyaxon will collect all artifacts and outputs that you share in the
     `plx-context/artifacts/run-uuid/outputs` to the default artifacts store
     that you configured during the deployment.
 
@@ -176,7 +197,7 @@ class V1Plugins(BaseConfig, polyaxon_sdk.V1Plugins):
     ```
 
     Sometimes you might want to access the artifacts path in your polyaxonfile,
-    Polyaxon expose a [context](/docs/core/specification/context/) that get resolved during
+    Polyaxon expose a [context](/docs/core//context/) that get resolved during
     the compilation time, you can just use
     "{{run_artifacts_path}}" global variable and it will be resolved automatically.
 
@@ -190,13 +211,13 @@ class V1Plugins(BaseConfig, polyaxon_sdk.V1Plugins):
 
     ```
 
-    For more information about the context, please check [context](/docs/core/specification/context/)
+    For more information about the context, please check [context](/docs/core/context/)
 
     ### collectLogs
 
     <blockquote class="light">This plugin is enabled by default.</blockquote>
 
-    By default Polyaxon will collect all logs related to your runs before deleting
+    By default, Polyaxon will collect all logs related to your runs before deleting
     the resource on the clusters. This ensures that your cluster(s) are kept clean and no resources
     are actively putting pressure on the API server.
 
@@ -213,7 +234,7 @@ class V1Plugins(BaseConfig, polyaxon_sdk.V1Plugins):
 
     <blockquote class="light">This plugin is enabled by default.</blockquote>
 
-    By default Polyaxon will collect all Mem/CPU/GPU resources
+    By default, Polyaxon will collect all Mem/CPU/GPU resources
     for your runs that use the python client.
 
     Sometimes you might want to avoid collecting this information for some runs,
@@ -230,7 +251,7 @@ class V1Plugins(BaseConfig, polyaxon_sdk.V1Plugins):
 
     <blockquote class="light">This plugin is enabled by default.</blockquote>
 
-    By default Polyaxon will resume from collecting metrics/outputs/artifacts
+    By default, Polyaxon will resume from collecting metrics/outputs/artifacts
     if a run fails and retries or if the user resume a run.
 
     To disable this plugin:
@@ -256,11 +277,22 @@ class V1Plugins(BaseConfig, polyaxon_sdk.V1Plugins):
     >>>   syncStatuses: false
     ```
 
+    ### externalHost
+    <blockquote class="light">Default is False.</blockquote>
+    In some edge cases where the auxiliaries and/or the main container cannot reach the API/Streams
+    services via the internal networking interface, you can enable this flag to tell Polyaxon
+    resolve the external host instead of the default behavior with the in-cluster host.
+
+    ```yaml
+    >>> plugins:
+    >>>   externalHost: true
+    ```
+
     ### logLevel
 
     <blockquote class="light">Default is None.</blockquote>
 
-    If you want to control the log level of your runs in similar way locally and on cluster,
+    If you want to control the log level of your runs in a similar way locally and on the cluster,
     you can either use env vars or this plugin to share the same log level with all containers
     running in your operation.
 
@@ -269,24 +301,16 @@ class V1Plugins(BaseConfig, polyaxon_sdk.V1Plugins):
     >>>   logLevel: warning
     ```
 
-    ### notifications
+    ### sidecar
 
     <blockquote class="light">Default is None.</blockquote>
 
-    Polyaxon used to only expose a global way for sending notifications about runs.
-
-    This plugin allows you to enable per run notification configuration.
-
-    For more information about notifications please check the
-    [notifications section](/docs/core/specification/notifications/)
+    To override the default global sidecar configuration.
 
     ```yaml
     >>> plugins:
-    >>>   notifications:
-    >>>     - connections: [slack-connection]
-    >>>       trigger: succeeded
-    >>>     - connections: [slack-connection, pagerduty-connection]
-    >>>       trigger: failed
+    >>>   sidecar:
+    >>>     syncInterval: 60
     ```
     """
 
@@ -296,10 +320,14 @@ class V1Plugins(BaseConfig, polyaxon_sdk.V1Plugins):
         "auth",
         "docker",
         "shm",
+        "mountArtifactsStore",
         "collectArtifacts",
         "collectLogs",
         "collectResources",
+        "autoResume",
         "syncStatuses",
+        "externalHost",
         "logLevel",
+        "sidecar",
         "notifications",
     ]
