@@ -65,18 +65,18 @@ class V1Dag(BaseConfig, polyaxon_sdk.V1Dag):
 
     Dags are defined in Polyaxon as a [component runtime](/docs/core/specification/component/#run),
     which makes them compatible with all knowledge used for running other runtimes:
-      * They can be defined in reusable components and can be registered in the Component Hub.
-      * They get executed using operations.
-      * They can be parametrized similar to jobs and services.
-      * Since they are defined as components' runtimes, and they run a graph of other components,
-        they can be nested natively.
-      * They can leverage all [pipeline helpers](/docs/automation/helpers/).
-      * They can run in parallel and can be used with [mapping](/docs/automation/mapping/) or
-        other [optimization algorithms](/docs/automation/optimization-engine/).
-      * They can run on [schedule](/docs/automation/schedules/)
-      * They can subscribe to [events](/docs/automation/events/)
-      * They can take advantage of all scheduling strategies to route operations to nodes,
-        namespaces, and clusters even within the same DAG.
+     * They can be defined in reusable components and can be registered in the Component Hub.
+     * They get executed using operations.
+     * They can be parametrized similar to jobs and services.
+     * Since they are defined as components' runtimes, and they run a graph of other components,
+       they can be nested natively.
+     * They can leverage all [pipeline helpers](/docs/automation/helpers/).
+     * They can run in parallel and can be used with [mapping](/docs/automation/mapping/) or
+       other [optimization algorithms](/docs/automation/optimization-engine/).
+     * They can run on [schedule](/docs/automation/schedules/)
+     * They can subscribe to [events](/docs/automation/events/)
+     * They can take advantage of all scheduling strategies to route operations to nodes,
+       namespaces, and clusters even within the same DAG.
 
     Args:
         kind: str, should be equal `dag`
@@ -85,7 +85,7 @@ class V1Dag(BaseConfig, polyaxon_sdk.V1Dag):
         environment: [V1Environment](/docs/core/specification/environment/), optional
         connections: List[str], optional
         volumes: List[[Kubernetes Volume](https://kubernetes.io/docs/concepts/storage/volumes/)],
-                 optional
+             optional
         concurrency: init, optional
         early_stopping: List[[EarlyStopping](/docs/automation/helpers/early-stopping)], optional
 
@@ -160,8 +160,8 @@ class V1Dag(BaseConfig, polyaxon_sdk.V1Dag):
     ```
 
     > **Note**: For more information about managing the execution graph
-    and creating dependencies between operations, please check the
-    [flow dependencies section](/docs/automation/flow-engine/flow-dependencies/).
+    > and creating dependencies between operations, please check the
+    > [flow dependencies section](/docs/automation/flow-engine/flow-dependencies/).
 
     ### references
 
@@ -172,7 +172,7 @@ class V1Dag(BaseConfig, polyaxon_sdk.V1Dag):
 
     Operations can reference components using:
         * [dagRef](/docs/core/specification/operation/#dagRef)
-            (reusable component defined inside the dag)
+             (reusable component defined inside the dag)
         * [hubRef](/docs/core/specification/operation/#hubRef)
         * [pathRef](/docs/core/specification/operation/#pathRef)
         * [urlRef](/docs/core/specification/operation/#urlRef)
@@ -462,14 +462,27 @@ class V1Dag(BaseConfig, polyaxon_sdk.V1Dag):
 
         return dags.sort_topologically(dag or self.dag, flatten=flatten)
 
-    def process_components(self, inputs=None):
+    def process_components(self, inputs=None, ignore_hub_validation: bool = False):
+        """`ignore_hub_validation` is currently used for ignoring validation
+        during tests with hub_ref.
+        """
         inputs = inputs or []
-        self._context["dag.name"] = V1IO(
-            name="name", type=types.STR, value="", is_optional=True
+
+        for g_context in contexts_sections.GLOBALS_CONTEXTS:
+            self._context[
+                "dag.{}.{}".format(contexts_sections.GLOBALS, g_context)
+            ] = V1IO(name=g_context, type=types.STR, value="", is_optional=True)
+
+        self._context["dag.{}".format(contexts_sections.INPUTS)] = V1IO(
+            name="inputs", type=types.DICT, value={}, is_optional=True
         )
-        self._context["dag.uuid"] = V1IO(
-            name="uuid", type=types.STR, value="", is_optional=True
+        self._context["dag.{}".format(contexts_sections.GLOBALS)] = V1IO(
+            name="globals", type=types.STR, value="", is_optional=True
         )
+        self._context["dag.{}".format(contexts_sections.ARTIFACTS)] = V1IO(
+            name="artifacts", type=types.STR, value="", is_optional=True
+        )
+
         for _input in inputs:
             self._context["dag.inputs.{}".format(_input.name)] = _input
 
@@ -492,8 +505,6 @@ class V1Dag(BaseConfig, polyaxon_sdk.V1Dag):
 
         for op in self.operations:
             op_name = op.name
-            if op.has_hub_reference:
-                continue
             if op.has_component_reference:
                 outputs = op.component.outputs
                 inputs = op.component.inputs
@@ -515,6 +526,8 @@ class V1Dag(BaseConfig, polyaxon_sdk.V1Dag):
                 self._op_component_mapping[op_name] = component_ref_name
                 outputs = self._components_by_names[component_ref_name].outputs
                 inputs = self._components_by_names[component_ref_name].inputs
+            elif op.has_hub_reference and ignore_hub_validation:
+                continue
             else:
                 raise PolyaxonSchemaError(
                     "Pipeline op has no definition field `{}`".format(op_name)
@@ -523,26 +536,34 @@ class V1Dag(BaseConfig, polyaxon_sdk.V1Dag):
             if outputs:
                 for output in outputs:
                     self._context[
-                        "ops.{}.outputs.{}".format(op_name, output.name)
+                        "ops.{}.{}.{}".format(
+                            op_name, contexts_sections.OUTPUTS, output.name
+                        )
                     ] = output
                     if output.type == types.ARTIFACTS:
                         self._context[
-                            "ops.{}.artifacts.{}".format(op_name, output.name)
+                            "ops.{}.{}.{}".format(
+                                op_name, contexts_sections.ARTIFACTS, output.name
+                            )
                         ] = output
 
             if inputs:
                 for cinput in inputs:
                     self._context[
-                        "ops.{}.inputs.{}".format(op_name, cinput.name)
+                        "ops.{}.{}.{}".format(
+                            op_name, contexts_sections.INPUTS, cinput.name
+                        )
                     ] = cinput
                     if cinput.type == types.ARTIFACTS:
                         self._context[
-                            "ops.{}.artifacts.{}".format(op_name, cinput.name)
+                            "ops.{}.{}.{}".format(
+                                op_name, contexts_sections.ARTIFACTS, cinput.name
+                            )
                         ] = cinput
             for g_context in contexts_sections.GLOBALS_CONTEXTS:
-                self._context["ops.{}.globals.{}".format(op_name, g_context)] = V1IO(
-                    name=g_context, type=types.STR, value="", is_optional=True
-                )
+                self._context[
+                    "ops.{}.{}.{}".format(op_name, contexts_sections.GLOBALS, g_context)
+                ] = V1IO(name=g_context, type=types.STR, value="", is_optional=True)
 
             # We allow to resolve name, status, project, all outputs/inputs, iteration
             self._context["ops.{}.{}".format(op_name, contexts_sections.INPUTS)] = V1IO(
@@ -562,9 +583,7 @@ class V1Dag(BaseConfig, polyaxon_sdk.V1Dag):
             ] = V1IO(name="io", type=types.STR, value={}, is_optional=True)
 
         for op in self.operations:
-            if op.has_hub_reference:
-                continue
-            elif op.has_component_reference:
+            if op.has_component_reference:
                 component_ref = op.definition.name
                 outputs = op.definition.outputs
                 inputs = op.definition.inputs
@@ -572,6 +591,8 @@ class V1Dag(BaseConfig, polyaxon_sdk.V1Dag):
                 component_ref = op.definition.name
                 outputs = self._components_by_names[component_ref].outputs
                 inputs = self._components_by_names[component_ref].inputs
+            elif op.has_hub_reference and ignore_hub_validation:
+                continue
             else:
                 raise PolyaxonSchemaError(
                     "Pipeline op has no definition field `{}`".format(op.name)
