@@ -1,6 +1,6 @@
 #!/usr/bin/python
 #
-# Copyright 2018-2020 Polyaxon, Inc.
+# Copyright 2018-2021 Polyaxon, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,6 +23,7 @@ from polyaxon.k8s import k8s_schemas
 from polyaxon.polyflow.environment import EnvironmentSchema
 from polyaxon.polyflow.init import InitSchema
 from polyaxon.polyflow.run.kinds import V1RunKind
+from polyaxon.polyflow.run.utils import DestinationImageMixin
 from polyaxon.schemas.base import BaseCamelSchema, BaseConfig
 from polyaxon.schemas.fields.swagger import SwaggerField
 
@@ -45,7 +46,7 @@ class JobSchema(BaseCamelSchema):
         return V1Job
 
 
-class V1Job(BaseConfig, polyaxon_sdk.V1Job):
+class V1Job(BaseConfig, DestinationImageMixin, polyaxon_sdk.V1Job):
     """Jobs are used to train machine learning models,
     process a dataset, execute generic tasks and can be used to perform a variety of functions
     from compiling a model to running an ETL operation.
@@ -55,10 +56,9 @@ class V1Job(BaseConfig, polyaxon_sdk.V1Job):
         environment: [V1Environment](/docs/core/specification/environment/), optional
         connections: List[str], optional
         volumes: List[[Kubernetes Volume](https://kubernetes.io/docs/concepts/storage/volumes/)],
-                 optional
+             optional
         init: List[[V1Init](/docs/core/specification/init/)], optional
-        sidecars: List[[sidecar containers](/docs/core/specification/sidecars/)],
-                  optional
+        sidecars: List[[sidecar containers](/docs/core/specification/sidecars/)], optional
         container: [Kubernetes Container](https://kubernetes.io/docs/concepts/containers/)
 
     ## YAML usage
@@ -161,7 +161,7 @@ class V1Job(BaseConfig, polyaxon_sdk.V1Job):
     >>>   volumes:
     >>>     - name: volume1
     >>>       persistentVolumeClaim:
-    >>>       claimName: pvc1
+    >>>         claimName: pvc1
     >>>   ...
     >>>   container:
     >>>     name: myapp-container
