@@ -1,6 +1,6 @@
 #!/usr/bin/python
 #
-# Copyright 2018-2020 Polyaxon, Inc.
+# Copyright 2018-2021 Polyaxon, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -26,17 +26,19 @@ from polyaxon.containers.names import INIT_PREFIX, SIDECAR_PREFIX
 from polyaxon.env_vars.keys import POLYAXON_KEYS_NO_API
 from polyaxon.exceptions import PolypodException
 from polyaxon.k8s import k8s_schemas
-from polyaxon.k8s.custom_resources.operation import get_resource_name, get_run_instance
 from polyaxon.polyflow import V1Environment, V1Init, V1Plugins
-from polyaxon.polypod.common.containers import (
-    ensure_container_name,
-    sanitize_container_command_args,
+from polyaxon.polypod.common.containers import ensure_container_name, sanitize_container
+from polyaxon.polypod.common.env_vars import (
+    get_base_env_vars,
+    get_env_var,
+    get_service_env_vars,
 )
-from polyaxon.polypod.common.env_vars import get_env_var, get_service_env_vars
 from polyaxon.polypod.common.mounts import get_mounts
 from polyaxon.polypod.init.artifacts import get_artifacts_path_container
 from polyaxon.polypod.init.auth import get_auth_context_container
+from polyaxon.polypod.init.custom import get_custom_init_container
 from polyaxon.polypod.init.dockerfile import get_dockerfile_init_container
+from polyaxon.polypod.init.file import get_file_init_container
 from polyaxon.polypod.init.git import get_git_init_container
 from polyaxon.polypod.init.store import get_store_container
 from polyaxon.polypod.main.container import get_main_container
@@ -47,23 +49,34 @@ from polyaxon.polypod.specs.replica import ReplicaSpec
 from polyaxon.schemas.types import V1ConnectionType, V1K8sResourceType
 from polyaxon.services.auth import AuthenticationTypes
 from polyaxon.services.headers import PolyaxonServiceHeaders, PolyaxonServices
+from polyaxon.utils.fqn_utils import get_resource_name, get_run_instance
+from polyaxon.utils.host_utils import get_api_host
 from polyaxon.utils.http_utils import clean_host
 from polyaxon.utils.list_utils import to_list
+from polyaxon.utils.sanitizers import sanitize_string_dict
+from polyaxon.utils.string_utils import slugify
 
 
 class ConverterAbstract:
-    def get_main_env_vars(self, **kwargs) -> Optional[List[k8s_schemas.V1EnvVar]]:
+    def get_main_env_vars(
+        self, external_host: bool = False, **kwargs
+    ) -> Optional[List[k8s_schemas.V1EnvVar]]:
         raise NotImplementedError
 
     def get_polyaxon_sidecar_service_env_vars(
         self,
+        external_host: bool = False,
     ) -> Optional[List[k8s_schemas.V1EnvVar]]:
         raise NotImplementedError
 
-    def get_auth_service_env_vars(self) -> Optional[List[k8s_schemas.V1EnvVar]]:
+    def get_auth_service_env_vars(
+        self, external_host: bool = False
+    ) -> Optional[List[k8s_schemas.V1EnvVar]]:
         raise NotImplementedError
 
-    def get_init_service_env_vars(self) -> Optional[List[k8s_schemas.V1EnvVar]]:
+    def get_init_service_env_vars(
+        self, external_host: bool = False
+    ) -> Optional[List[k8s_schemas.V1EnvVar]]:
         raise NotImplementedError
 
 
@@ -72,7 +85,7 @@ class BaseConverter(ConverterAbstract):
     GROUP = None
     API_VERSION = None
     PLURAL = None
-    K8S_LABELS_NAME = None
+    K8S_ANNOTATIONS_KIND = None
     K8S_LABELS_COMPONENT = None
     K8S_LABELS_PART_OF = None
     MAIN_CONTAINER_ID = None
@@ -127,9 +140,9 @@ class BaseConverter(ConverterAbstract):
             raise PolypodException(
                 "Please make sure that a spawner subclass has a valid PLURAL"
             )
-        if not self.K8S_LABELS_NAME:
+        if not self.K8S_ANNOTATIONS_KIND:
             raise PolypodException(
-                "Please make sure that a spawner subclass has a valid K8S_LABELS_NAME"
+                "Please make sure that a spawner subclass has a valid K8S_ANNOTATIONS_KIND"
             )
         if not self.K8S_LABELS_COMPONENT:
             raise PolypodException(
@@ -142,9 +155,9 @@ class BaseConverter(ConverterAbstract):
 
     def get_recommended_labels(self, version: str):
         return {
-            "app.kubernetes.io/name": self.run_name[:63]
-            if self.run_name
-            else self.run_name,
+            "app.kubernetes.io/name": slugify(
+                self.run_name[:63] if self.run_name else self.run_name
+            ),
             "app.kubernetes.io/instance": self.run_uuid,
             "app.kubernetes.io/version": version,
             "app.kubernetes.io/part-of": self.K8S_LABELS_PART_OF,
@@ -158,32 +171,46 @@ class BaseConverter(ConverterAbstract):
             "operation.polyaxon.com/name": self.run_name,
             "operation.polyaxon.com/owner": self.owner_name,
             "operation.polyaxon.com/project": self.project_name,
+            "operation.polyaxon.com/kind": self.K8S_ANNOTATIONS_KIND,
         }
 
+    def get_annotations(self, annotations: Dict):
+        annotations = annotations or {}
+        annotations = copy.copy(annotations)
+        annotations.update(self.annotations)
+        return sanitize_string_dict(annotations)
+
     def get_labels(self, version: str, labels: Dict):
         labels = labels or {}
         labels = copy.copy(labels)
         labels.update(self.get_recommended_labels(version=version))
-        return labels
+        return sanitize_string_dict(labels)
 
     @staticmethod
     def get_by_name(values: List[Any]):
         return {c.name: c for c in values}
 
-    def get_main_env_vars(self, **kwargs) -> Optional[List[k8s_schemas.V1EnvVar]]:
-        return None
+    def get_main_env_vars(
+        self, external_host: bool = False, **kwargs
+    ) -> Optional[List[k8s_schemas.V1EnvVar]]:
+        return get_base_env_vars()
 
     def get_polyaxon_sidecar_service_env_vars(
         self,
+        external_host: bool = False,
     ) -> Optional[List[k8s_schemas.V1EnvVar]]:
         if settings.CLIENT_CONFIG.no_api:
             return [get_env_var(name=POLYAXON_KEYS_NO_API, value=True)]
         return None
 
-    def get_auth_service_env_vars(self) -> Optional[List[k8s_schemas.V1EnvVar]]:
+    def get_auth_service_env_vars(
+        self, external_host: bool = False
+    ) -> Optional[List[k8s_schemas.V1EnvVar]]:
         return None
 
-    def get_init_service_env_vars(self) -> Optional[List[k8s_schemas.V1EnvVar]]:
+    def get_init_service_env_vars(
+        self, external_host: bool = False
+    ) -> Optional[List[k8s_schemas.V1EnvVar]]:
         if settings.CLIENT_CONFIG.no_api:
             return [get_env_var(name=POLYAXON_KEYS_NO_API, value=True)]
         return None
@@ -202,7 +229,9 @@ class BaseConverter(ConverterAbstract):
         kv_env_vars: List[List] = None,
         ports: List[int] = None,
     ) -> k8s_schemas.V1Container:
-        env = self.get_main_env_vars()
+        env = self.get_main_env_vars(
+            external_host=contexts.external_host if contexts else False
+        )
         volume_mounts = get_mounts(
             use_auth_context=contexts.auth,
             use_artifacts_context=False,  # Main container has a check and handling for this
@@ -242,14 +271,16 @@ class BaseConverter(ConverterAbstract):
         polyaxon_sidecar_container = get_sidecar_container(
             container_id=self.MAIN_CONTAINER_ID,
             polyaxon_sidecar=polyaxon_sidecar,
-            env=self.get_polyaxon_sidecar_service_env_vars(),
+            env=self.get_polyaxon_sidecar_service_env_vars(
+                external_host=contexts.external_host if contexts else False
+            ),
             artifacts_store=artifacts_store,
             contexts=contexts,
             run_path=self.run_path,
         )
         containers = to_list(polyaxon_sidecar_container, check_none=True)
         containers += sidecar_containers
-        return [sanitize_container_command_args(c) for c in containers]
+        return [sanitize_container(c) for c in containers]
 
     def handle_init_connections(
         self,
@@ -260,34 +291,72 @@ class BaseConverter(ConverterAbstract):
         contexts: PluginsContextsSpec,
     ) -> List[k8s_schemas.V1Container]:
         containers = []
+        external_host = contexts.external_host if contexts else False
 
-        # Prepare connections that Polyaxon can inti automatically
+        # Prepare connections that Polyaxon can init automatically
         for init_connection in init_connections:
             if init_connection.connection:
                 connection_spec = connection_by_names.get(init_connection.connection)
-                if init_connection.git:  # Update the default schema
-                    connection_spec.schema.patch(init_connection.git)
-                if V1ConnectionKind.is_git(connection_spec.kind):
+                # Handling ssh with git
+                if (
+                    V1ConnectionKind.is_ssh(connection_spec.kind)
+                    and init_connection.git
+                ):
+                    connection_spec.schema.patch_git(init_connection.git)
                     containers.append(
                         get_git_init_container(
                             polyaxon_init=polyaxon_init,
                             connection=connection_spec,
                             container=init_connection.container,
-                            env=self.get_init_service_env_vars(),
+                            env=self.get_init_service_env_vars(
+                                external_host=external_host
+                            ),
                             mount_path=init_connection.path,
                             contexts=contexts,
                             track=True,
                         )
                     )
-                if V1ConnectionKind.is_artifact(connection_spec.kind):
+                elif V1ConnectionKind.is_git(connection_spec.kind):
+                    if init_connection.git:  # Update the default schema
+                        connection_spec.schema.patch(init_connection.git)
+                    containers.append(
+                        get_git_init_container(
+                            polyaxon_init=polyaxon_init,
+                            connection=connection_spec,
+                            container=init_connection.container,
+                            env=self.get_init_service_env_vars(
+                                external_host=external_host
+                            ),
+                            mount_path=init_connection.path,
+                            contexts=contexts,
+                            track=True,
+                        )
+                    )
+                elif V1ConnectionKind.is_artifact(connection_spec.kind):
                     containers.append(
                         get_store_container(
                             polyaxon_init=polyaxon_init,
                             connection=connection_spec,
                             artifacts=init_connection.artifacts,
                             container=init_connection.container,
-                            env=self.get_init_service_env_vars(),
+                            env=self.get_init_service_env_vars(
+                                external_host=external_host
+                            ),
                             mount_path=init_connection.path,
+                            is_default_artifacts_store=artifacts_store
+                            and init_connection.connection == artifacts_store.name,
+                        )
+                    )
+                else:
+                    containers.append(
+                        get_custom_init_container(
+                            connection=connection_spec,
+                            container=init_connection.container,
+                            env=self.get_init_service_env_vars(
+                                external_host=external_host
+                            ),
+                            mount_path=init_connection.path,
+                            contexts=contexts,
                         )
                     )
             else:
@@ -299,8 +368,11 @@ class BaseConverter(ConverterAbstract):
                             connection=artifacts_store,
                             artifacts=init_connection.artifacts,
                             container=init_connection.container,
-                            env=self.get_init_service_env_vars(),
+                            env=self.get_init_service_env_vars(
+                                external_host=external_host
+                            ),
                             mount_path=init_connection.path,
+                            is_default_artifacts_store=True,
                         )
                     )
                 # git init without connection
@@ -316,7 +388,9 @@ class BaseConverter(ConverterAbstract):
                                 secret=None,
                             ),
                             container=init_connection.container,
-                            env=self.get_init_service_env_vars(),
+                            env=self.get_init_service_env_vars(
+                                external_host=external_host
+                            ),
                             mount_path=init_connection.path,
                             contexts=contexts,
                             track=False,
@@ -328,7 +402,24 @@ class BaseConverter(ConverterAbstract):
                         get_dockerfile_init_container(
                             polyaxon_init=polyaxon_init,
                             dockerfile_args=init_connection.dockerfile,
-                            env=self.get_init_service_env_vars(),
+                            env=self.get_init_service_env_vars(
+                                external_host=external_host
+                            ),
+                            mount_path=init_connection.path,
+                            contexts=contexts,
+                            run_path=self.run_path,
+                            run_instance=self.run_instance,
+                        )
+                    )
+                # File initialization
+                if init_connection.file:
+                    containers.append(
+                        get_file_init_container(
+                            polyaxon_init=polyaxon_init,
+                            file_args=init_connection.file,
+                            env=self.get_init_service_env_vars(
+                                external_host=external_host
+                            ),
                             mount_path=init_connection.path,
                             contexts=contexts,
                             run_path=self.run_path,
@@ -358,7 +449,10 @@ class BaseConverter(ConverterAbstract):
         if contexts and contexts.auth:
             containers.append(
                 get_auth_context_container(
-                    polyaxon_init=polyaxon_init, env=self.get_auth_service_env_vars()
+                    polyaxon_init=polyaxon_init,
+                    env=self.get_auth_service_env_vars(
+                        external_host=contexts.external_host
+                    ),
                 )
             )
 
@@ -382,7 +476,7 @@ class BaseConverter(ConverterAbstract):
             contexts=contexts,
         )
         init_containers = containers + init_containers
-        return [sanitize_container_command_args(c) for c in init_containers]
+        return [sanitize_container(c) for c in init_containers]
 
     def filter_containers_from_init(
         self, init: List[V1Init]
@@ -416,7 +510,9 @@ class BaseConverter(ConverterAbstract):
         connections = connections or []
         environment = environment or V1Environment()
         environment.service_account_name = (
-            environment.service_account_name or default_sa
+            environment.service_account_name
+            or default_sa
+            or settings.AGENT_CONFIG.runs_sa
         )
 
         init_connections = self.filter_connections_from_init(init=init)
@@ -425,6 +521,7 @@ class BaseConverter(ConverterAbstract):
             contexts=contexts,
             artifacts_store=artifacts_store,
             init_connections=init_connections,
+            connections=connections,
             connection_by_names=connection_by_names,
             secrets=secrets,
             config_maps=config_maps,
@@ -461,12 +558,14 @@ class BaseConverter(ConverterAbstract):
         )
 
         labels = self.get_labels(version=pkg.VERSION, labels=environment.labels)
+        annotations = self.get_annotations(annotations=environment.annotations)
         return ReplicaSpec(
             volumes=volumes,
             init_containers=init_containers,
             sidecar_containers=sidecar_containers,
             main_container=main_container,
             labels=labels,
+            annotations=annotations,
             environment=environment,
             num_replicas=num_replicas,
         )
@@ -476,6 +575,12 @@ class BaseConverter(ConverterAbstract):
 
 
 class PlatformConverterMixin(ConverterAbstract):
+    def get_api_host(self, external_host: bool = False):
+        if external_host:
+            return get_api_host(default=settings.CLIENT_CONFIG.host)
+        else:
+            return clean_host(settings.CLIENT_CONFIG.host)
+
     def get_service_env_vars(
         self,
         service_header: str,
@@ -484,6 +589,7 @@ class PlatformConverterMixin(ConverterAbstract):
         include_internal_token: bool = False,
         include_agent_token: bool = False,
         authentication_type: str = None,
+        external_host: bool = False,
     ) -> List[k8s_schemas.V1EnvVar]:
         header = header or PolyaxonServiceHeaders.SERVICE
         return get_service_env_vars(
@@ -495,15 +601,20 @@ class PlatformConverterMixin(ConverterAbstract):
             include_agent_token=include_agent_token,
             polyaxon_default_secret_ref=settings.AGENT_CONFIG.app_secret_name,
             polyaxon_agent_secret_ref=settings.AGENT_CONFIG.agent_secret_name,
-            api_host=clean_host(settings.CLIENT_CONFIG.host),
+            api_host=self.get_api_host(external_host),
             api_version=VERSION_V1,
             run_instance=self.run_instance,
         )
 
-    def get_main_env_vars(self, **kwargs):
-        return self.get_service_env_vars(service_header=PolyaxonServices.RUNNER)
+    def get_main_env_vars(self, external_host: bool = False, **kwargs):
+        return self.get_service_env_vars(
+            service_header=PolyaxonServices.RUNNER,
+            external_host=external_host,
+        )
 
-    def get_auth_service_env_vars(self) -> List[k8s_schemas.V1EnvVar]:
+    def get_auth_service_env_vars(
+        self, external_host: bool = False
+    ) -> List[k8s_schemas.V1EnvVar]:
         return self.get_service_env_vars(
             service_header=PolyaxonServices.INITIALIZER,
             include_internal_token=self.internal_auth,
@@ -518,18 +629,25 @@ class PlatformConverterMixin(ConverterAbstract):
                 if self.internal_auth
                 else PolyaxonServiceHeaders.SERVICE
             ),
+            external_host=external_host,
         )
 
-    def get_polyaxon_sidecar_service_env_vars(self) -> List[k8s_schemas.V1EnvVar]:
+    def get_polyaxon_sidecar_service_env_vars(
+        self, external_host: bool = False
+    ) -> List[k8s_schemas.V1EnvVar]:
         return self.get_service_env_vars(
             service_header=PolyaxonServices.SIDECAR,
             authentication_type=AuthenticationTypes.TOKEN,
             header=PolyaxonServiceHeaders.SERVICE,
+            external_host=external_host,
         )
 
-    def get_init_service_env_vars(self) -> List[k8s_schemas.V1EnvVar]:
+    def get_init_service_env_vars(
+        self, external_host: bool = False
+    ) -> List[k8s_schemas.V1EnvVar]:
         return self.get_service_env_vars(
             service_header=PolyaxonServices.INITIALIZER,
             authentication_type=AuthenticationTypes.TOKEN,
             header=PolyaxonServiceHeaders.SERVICE,
+            external_host=external_host,
         )
