@@ -1,6 +1,6 @@
 #!/usr/bin/python
 #
-# Copyright 2018-2020 Polyaxon, Inc.
+# Copyright 2018-2021 Polyaxon, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,9 +16,11 @@
 
 from typing import Dict, Iterable, Optional
 
-from polyaxon.polyflow import V1CompiledOperation, V1Job, V1Plugins  # noqa
-from polyaxon.polypod.compiler.converters import BaseConverter
-from polyaxon.polypod.compiler.converters.base import PlatformConverterMixin
+from polyaxon.polyflow import V1CompiledOperation, V1Job, V1Plugins
+from polyaxon.polypod.compiler.converters.base import (
+    BaseConverter,
+    PlatformConverterMixin,
+)
 from polyaxon.polypod.custom_resources import get_job_custom_resource
 from polyaxon.polypod.mixins import JobMixin
 from polyaxon.polypod.specs.contexts import PluginsContextsSpec
@@ -33,6 +35,7 @@ class JobConverter(JobMixin, BaseConverter):
         connection_by_names: Dict[str, V1ConnectionType],
         secrets: Optional[Iterable[V1K8sResourceType]],
         config_maps: Optional[Iterable[V1K8sResourceType]],
+        default_sa: str = None,
         default_auth: bool = False,
     ) -> Dict:
         job = compiled_operation.run  # type: V1Job
@@ -51,6 +54,7 @@ class JobConverter(JobMixin, BaseConverter):
             connection_by_names=connection_by_names,
             secrets=secrets,
             config_maps=config_maps,
+            default_sa=default_sa,
         )
         return get_job_custom_resource(
             namespace=self.namespace,
@@ -65,6 +69,7 @@ class JobConverter(JobMixin, BaseConverter):
             sync_statuses=contexts.sync_statuses,
             notifications=plugins.notifications,
             labels=replica_spec.labels,
+            annotations=replica_spec.annotations,
         )
 
 
