@@ -1,6 +1,6 @@
 #!/usr/bin/python
 #
-# Copyright 2018-2020 Polyaxon, Inc.
+# Copyright 2018-2021 Polyaxon, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -28,7 +28,7 @@ from polyaxon.proxies.schemas.urls import (
     get_ssl_server_name,
 )
 
-OPTIONS = """
+OPTIONS = r"""
 location {path} {{
     {auth}
     {resolver}
@@ -66,7 +66,8 @@ def get_api_config(
 
 def get_api_location_config(resolver: str, auth=str):
     service = get_service_url(
-        host=settings.PROXIES_CONFIG.api_host, port=settings.PROXIES_CONFIG.api_port,
+        host=settings.PROXIES_CONFIG.api_host,
+        port=settings.PROXIES_CONFIG.api_port,
     )
     if not settings.PROXIES_CONFIG.api_use_resolver:
         resolver = ""
