@@ -1,6 +1,6 @@
 #!/usr/bin/python
 #
-# Copyright 2018-2020 Polyaxon, Inc.
+# Copyright 2018-2021 Polyaxon, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,7 +18,7 @@ import sys
 
 from contextlib import contextmanager
 
-from polyaxon.utils.constants import NEWLINES
+from polyaxon.utils.cli_constants import NEWLINES
 
 INDENT_CHAR = " "
 INDENT_STRINGS = []
